#include <sys/wait.h>
#include <stdbool.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <spawn.h>
#include <stdio.h>

#include "error.h"
#include "util.h"
#include "run.h"

extern char **environ;

char *run_find_cmd(char *cmd){
  char *path = getenv("PATH"), *save, *el;

  while((el = strtok_r(path, ":", &save)) != NULL){
    char fp[strlen(el)+strlen(cmd)+2];
    join(fp, el, cmd);

    if(exists(fp))
      return strdup(fp);
  }
  return NULL;
}

bool run_cmd(char **all){
  pid_t pid;
  if(posix_spawn(&pid, all[0], NULL, NULL, all, environ) != 0)
    return false;

  int status;
  waitpid(pid, &status, 0);
  return status == 0;
}

bool run_cmd_root(char *cmd, char **args){
  char *rootcmd = NULL;

  if(NULL == run_find_cmd(cmd)){
    error_set("Command not found");
    errno = RunCmdNotFound;
    return false;
  }

  if(getuid()==0)
    goto RUN;
  else if((rootcmd = run_find_cmd("doas")) != NULL)
    goto RUN;
  else if((rootcmd = run_find_cmd("sudo" )) != NULL)
    goto RUN;

  error_set("There is no doas or sudo installed");
  errno = RunNoRoot;
  return false;

RUN:
  int arg_count = 0;
  while(args[arg_count] != NULL)
    arg_count++;
  arg_count += 2;

  if(rootcmd != NULL)
    arg_count++;

  char **all = malloc(arg_count*sizeof(char*));
  int indx = 0;

  if(rootcmd != NULL){
    all[indx] = rootcmd;
    all[++indx] = cmd;
  }else
    all[indx] = rootcmd;

  for(int i = 0; i < arg_count; i++)
    all[++indx] = args[i];
  all[++indx] = NULL;

  bool ret = true;

  if(!run_cmd(all)){
    error_set("Command failed");
    errno = RunCmdFail;
    ret = false;
  }

  free(all);
  return ret;
}
